﻿//*****************************************************************************
//
//  ColorComboBox140.cpp
//
//*****************************************************************************

// これはMFCのCComboBoxクラスから派生したCColorComboBoxクラスを定義します。
// ■CWnd
// └■CComboBox
// 　└■CColorComboBox
// これはMicrosoft Visual C++ 2015 Community Editionで作られています。
// このライブラリは、GNU 劣等一般公衆利用許諾契約書(LGPL)に基づき配布されます。
// プロジェクトホームページ：http://www5a.biglobe.ne.jp/~kuzunet/ 

// This library is free software; you can redistribute it and/or 
// modify it under the terms of the GNU Lesser General Public 
// License as published by the Free Software Foundation; either 
// version 2.1 of the License, or (at your option) any later version. 

// This library is distributed in the hope that it will be useful, 
// but WITHOUT ANY WARRANTY; without even the implied warranty of 
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
// Lesser General Public License for more details. 

// You should have received a copy of the GNU Lesser General Public 
// License along with this library; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 

#include <afxwin.h>
#include <afxext.h>
#include <afxdllx.h>
#include "ColorComboBox140.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

// DLLの初期化(AppWizardによる自動生成)
// これはMFCの拡張DLLである。拡張DLLの初期化方法については下記のサイトを参照。
// http://www.microsoft.com/japan/msdn/library/default.asp?
// url=/japan/msdn/library/ja/vccore/html/_core_initialize_extension_dlls.asp
// #inlcude <afxdllx.h>を忘れぬこと。

extern "C" 

static AFX_EXTENSION_MODULE ColorComboBox140DLL = { NULL, NULL };

int APIENTRY
DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID lpReserved)
{
	if (dwReason == DLL_PROCESS_ATTACH)
	{
		TRACE0("COLORCOMBOBOX140.DLL 初期化処理中。\n");
		
		// 拡張 DLL を１回だけ初期化します。
		AfxInitExtensionModule(ColorComboBox140DLL, hInstance);

		// この DLL をリソース チェインへ挿入します。
		new CDynLinkLibrary(ColorComboBox140DLL);
	}
	else if (dwReason == DLL_PROCESS_DETACH)
	{
		TRACE0("COLORCOMBOBOX140.DLL 終了処理中。\n");
	}
	return 1;   // OK
}


//*****************************************************************************
//
// CColorComboBoxクラス
//
//*****************************************************************************

// メッセージマップ
IMPLEMENT_DYNCREATE (CColorComboBox, CComboBox)

BEGIN_MESSAGE_MAP (CColorComboBox, CComboBox)
	ON_CONTROL_REFLECT (CBN_SELCHANGE, OnSelChange)
END_MESSAGE_MAP ()

// オーバーライド

// ウィンドウスタイルの変更
BOOL CColorComboBox::PreCreateWindow (CREATESTRUCT& cs) {
	if (!CComboBox::PreCreateWindow (cs)) {
		return FALSE;
	}
	cs.dwExStyle |= WS_EX_CLIENTEDGE;
	cs.style &= ~(CBS_DROPDOWN | CBS_SIMPLE | CBS_OWNERDRAWVARIABLE | CBS_SORT);
	cs.style |= (CBS_DROPDOWNLIST | CBS_OWNERDRAWFIXED | CBS_HASSTRINGS);
	return TRUE;
}

// 項目の高さ
void CColorComboBox::MeasureItem (LPMEASUREITEMSTRUCT lpmis) {
	lpmis->itemHeight = 14;
}

// 項目の描画
void CColorComboBox::DrawItem (LPDRAWITEMSTRUCT lpdis) {
	CDC dc;
	dc.Attach (lpdis->hDC);
	CRect rect = lpdis->rcItem;
	int nIndex = lpdis->itemID;

	CBrush* pBrush = new CBrush;
	pBrush->CreateSolidBrush (::GetSysColor ((lpdis->itemState &
		ODS_SELECTED) ? COLOR_HIGHLIGHT : COLOR_WINDOW));
	dc.FillRect (rect, pBrush);
	delete pBrush;

	if (lpdis->itemState & ODS_FOCUS) {
		dc.DrawFocusRect (rect);
	}

	if (nIndex != (UINT)-1) {
		COLORREF clrSample = (COLORREF)GetItemData (nIndex);
		if (clrSample == COLORCOMBOBOX_OTHER) {
			CFont* pFont = CFont::FromHandle 
				((HFONT)(::GetStockObject (DEFAULT_GUI_FONT)));
			CFont* pOldFont = dc.SelectObject (pFont);
			dc.SetBkMode (TRANSPARENT);
			dc.DrawText (_T("その他..."), rect, DT_LEFT | DT_VCENTER | DT_SINGLELINE);
			dc.SelectObject (pOldFont);
		}
		else {
			CRect rcSample (rect);
			rcSample.DeflateRect (3, 3);
			dc.FillSolidRect (rcSample, clrSample);
		}
	}
	else {
		CRect rcSample (rect);
		rcSample.DeflateRect (3, 3);
		dc.FillSolidRect (rcSample, m_clrCurColor);		
	}
	dc.Detach ();
}

// オペレーション

// 現在の色を設定する(この関数はいつでも使える。)
BOOL CColorComboBox::SetCurColor (COLORREF clr) {
	if (clr < 0x00000000 || clr > 0x00FFFFFF) {
		return FALSE;
	}
	m_clrCurColor = clr;
	if (m_hWnd != NULL) {
		int i = 0;
		int nCount = GetCount ();
		// 一致するカラーサンプルを選択状態にする
		for (i = 0; i <= nCount; i++) {
			COLORREF clrSampleColor = GetItemData (i);
			if (clrSampleColor == m_clrCurColor) {
				SetCurSel (i);
				break;
			}
		}
		// 一致するカラーサンプルがなかったので選択解除
		if (i >= nCount) {
			SetCurSel (-1);
		}
		Invalidate ();
	}
	return TRUE;
}

// すべてのカラーサンプルをクリアする。
void CColorComboBox::ClearSampleColor () {
	ResetContent ();
}

// カラーサンプルを追加する。
int CColorComboBox::AddSampleColor (COLORREF clr) {
	int nIndex = AddString (_T(""));
	if (nIndex == CB_ERR || nIndex == CB_ERRSPACE) {
		return nIndex;
	}
	return SetSampleColor (nIndex , (DWORD)clr);
}

// カラーサンプルを取得する。
COLORREF CColorComboBox::GetSampleColor (int nIndex) const {
	return (COLORREF)GetItemData (nIndex);
}

// カラーサンプルを設定する。
int CColorComboBox::SetSampleColor (int nIndex, COLORREF clr) {
	int nRet = SetItemData (nIndex, (DWORD)clr);
	int nCurSel = GetCurSel ();
	// 新しいカラーサンプルとカレントカラーの一致チェック
	if (nCurSel == CB_ERR || nCurSel == nIndex) { 
		COLORREF clrSampleColor = GetItemData (nIndex);
		// 新しいカラーサンプルとカレントカラーは一致したので選択
		if (clrSampleColor == m_clrCurColor) {
			SetCurSel (nIndex);
		}
		// 新しいカラーサンプルとカレントカラーは一致しなかったので選択解除
		else {
			SetCurSel (-1);
		}
	}
	return nRet;
}

// メッセージマップ

// カレントセルが変更されたとき(メッセージリフレクション)
void CColorComboBox::OnSelChange () {
	int nCurSel = GetCurSel ();
	if (0 <= nCurSel && nCurSel < GetCount ()) {
		COLORREF clrCurColor = (COLORREF)GetItemData (nCurSel);
		if (clrCurColor == COLORCOMBOBOX_OTHER) {
			CColorDialog theDlg;
			theDlg.m_cc.Flags |= CC_FULLOPEN;
			if (theDlg.DoModal () == IDOK) {
				m_clrCurColor = theDlg.GetColor ();
			}
			SetCurSel (-1);
		}
		else {
			m_clrCurColor = clrCurColor;
		}
		Invalidate ();
	}
}
